# -*- coding: utf-8 -*-

"""
An instance of the logger. Wraps the Python logger and some other 3rd-P integrations.
"""

import sys
from .dict_expander import expand
from .color import Color


class Logger:

    COLUMN_PADDING = 2  # Minimum width of column when writing to console.

    # Drawing the dictionaries.
    BOX_STEM = "├"
    BOX_STEM_END = "└"
    BOX_BRANCH = "─"
    BOX_BRANCH_DOWN = "┬"
    LOG_BULLET = "⚫"  # "┃"

    # Log level constants.
    LEVEL_DEBUG: int = 10
    LEVEL_INFO: int = 20
    LEVEL_WARNING: int = 30
    LEVEL_ERROR: int = 40
    LEVEL_CRITICAL: int = 50

    LEVEL_NAME = {
        LEVEL_INFO: "INFO",
        LEVEL_WARNING: "WARNING",
        LEVEL_ERROR: "ERROR",
        LEVEL_CRITICAL: "CRITICAL",
        LEVEL_DEBUG: "DEBUG"
    }

    LEVEL_COLOR = {
        LEVEL_INFO: Color.BLUE,
        LEVEL_WARNING: Color.YELLOW,
        LEVEL_ERROR: Color.RED,
        LEVEL_CRITICAL: Color.RED,
        LEVEL_DEBUG: Color.GREEN,
    }

    # ======================================================================================================================
    # Singleton Access
    # ======================================================================================================================

    _instance = None

    @staticmethod
    def get_instance() -> "Logger":
        if Logger._instance is None:
            Logger._instance = Logger()
        return Logger._instance

    @staticmethod
    def format_log_name(x: str) -> str:
        return f"[{x}]"

    def __init__(self):
        # Native Python logging module.
        self.file_logger = None
        self.file_logging_map = None

        self.with_color = True
        self.with_level_prefix = True

        self.index: int = 1

    # ======================================================================================================================
    # Loading and Initialization
    # ======================================================================================================================

    def write(self, message, data, level: int):
        # Parse the data first.
        if data is not None:
            if type(data) is not dict:
                message = f"{message}: {data}"
                data = None

        self.console_write(message, data, level, with_color=self.with_color)

    def console_write(self, message, data: dict, level: int, with_color: bool = False):
        """ Custom function to write message to console. """

        # Write the Header.
        self.console_write_line(message, level, with_color)

        # Write the Items.
        if data is not None:
            stem_color = None if not with_color else self.LEVEL_COLOR[level]
            expanded_lines = expand(key=None, data=data, stem_color=stem_color)
            for x in expanded_lines:
                print(x)
            sys.stdout.flush()

    def console_write_line(self, content, level, with_color: bool = False):

        prefix = self.format_log_name(self.LEVEL_NAME[level])
        pad_length = max(0, self.COLUMN_PADDING - len(prefix))
        prefix += " " * pad_length
        self.index += 1

        if with_color:
            if level > self.LEVEL_INFO:
                content = "{} {}".format(prefix, content)
                content = self.set_level_color(content, level)
            else:
                prefix = self.set_level_color(prefix, level)
                content = "{} {}".format(prefix, content)
        else:
            content = "{} {}".format(prefix, content)

        print(content)
        sys.stdout.flush()

    def set_level_color(self, content, level):

        content = str(content)
        level_color = self.LEVEL_COLOR[level]
        return Color.set_color(level_color, content)



from typing import List, Union
from .color import Color


class DictExpander:

    # Drawing the dictionaries.
    BOX_STEM = "├"
    BOX_STEM_END = "└"
    BOX_BRANCH = "─"
    BOX_BRANCH_DOWN = "┬"

    def __init__(self):
        pass

    @classmethod
    def recursive_expansion(
            cls, data: dict, output_list, stem_color: Union[str, None], value_color: Union[str, None],
            indent: int = 0, indent_end_stack: list = []):

        if data is not None:
            n_elements = 0
            for k, v in data.items():

                n_elements += 1
                use_key = str(k)
                is_last_element = not (n_elements < len(data))
                element_is_populated_dict = type(v) is dict and len(v) > 0

                stem_arr = []
                for i in range(indent):
                    stem_arr.append("│ " if not indent_end_stack[i] else "  ")

                stem_arr.append(cls.BOX_STEM_END if is_last_element else cls.BOX_STEM)
                stem_arr.append(cls.BOX_BRANCH)

                if element_is_populated_dict:
                    stem_arr.append(cls.BOX_BRANCH_DOWN)
                else:
                    stem_arr.append(cls.BOX_BRANCH)

                stem = "".join(stem_arr)

                if not element_is_populated_dict:
                    use_key += ":"  # Add a colon to the key before applying color.

                if stem_color is not None:
                    use_key = Color.set_color(stem_color, use_key)
                    stem = Color.set_color(stem_color, stem)

                if element_is_populated_dict:
                    line = f"  {stem} {use_key}"
                    output_list.append(line)
                    indent_end_stack.append(is_last_element)
                    cls.recursive_expansion(v, output_list, stem_color, value_color, indent + 1, indent_end_stack)
                    indent_end_stack.pop()
                else:

                    if type(v) == list:
                        # So lists appear without the quotes.
                        list_string = ", ".join([str(x) for x in v])
                        data_string = f"[{list_string}]"
                    else:
                        data_string = str(v)

                    if value_color is not None:
                        data_string = Color.set_color(value_color, data_string)
                    line = f"  {stem} {use_key} {data_string}"
                    output_list.append(line)

# ======================================================================================================================
# Expand the object.
# ======================================================================================================================


def expand(key: Union[str, None], data: dict,
           stem_color: Union[str, None] = None,
           value_color: Union[str, None] = None) -> List[str]:
    output = []
    if key is not None:
        # Convert key to string.
        key = str(key)

        # Color the key if needed.
        if stem_color is not None:
            key = Color.set_color(stem_color, key)

        # Add it to the display list and begin expansion.
        output.append(key)

    DictExpander.recursive_expansion(data, output, stem_color, value_color)
    return output



# -*- coding: utf-8 -*-

"""
Static functions for using the logger. The logger is supposed to be set up
with lazy loading, and to be configured via the generated .env file.
"""

from .logger import Logger


def debug(message, data=None):
    __log_with_level(message, data, Logger.LEVEL_DEBUG)


def info(message, data=None):
    __log_with_level(message, data, Logger.LEVEL_INFO)


def warning(message, data=None):
    __log_with_level(message, data, Logger.LEVEL_WARNING)


def error(message, data=None):
    __log_with_level(message, data, Logger.LEVEL_ERROR)


def critical(message, data=None):
    __log_with_level(message, data, Logger.LEVEL_CRITICAL)


def get_instance():
    return Logger.get_instance()


def __log_with_level(message, data, level):
    logger = Logger.get_instance()
    logger.write(message, data, level)




class Color:

    RED: str = "RED"
    BLUE: str = "BLUE"
    GREEN: str = "GREEN"
    YELLOW: str = "YELLOW"
    PURPLE: str = "PURPLE"
    BLACK: str = "BLACK"
    DEFAULT: str = "DEFAULT"

    _ESC_CHAR = ""  # This is not a blank char, this is a unicode /33.

    _COLOR_MAP = {
        RED: "[31m",
        GREEN: "[32m",
        YELLOW: "[33m",
        BLUE: "[34m",
        PURPLE: "[35m",
        BLACK: "[90m",
        DEFAULT: "[0m",
    }

    @classmethod
    def set_color(cls, key: str, content: str):
        start_color = Color._ESC_CHAR + cls._COLOR_MAP[key]
        end_color = Color._ESC_CHAR + cls._COLOR_MAP[cls.DEFAULT]
        return f"{start_color}{content}{end_color}"

    # ======================================================================================================================
    # Public Set Color Methods
    # ======================================================================================================================


def red(text: str) -> str:
    return Color.set_color(Color.RED, text)


def blue(text: str) -> str:
    return Color.set_color(Color.BLUE, text)


def green(text: str) -> str:
    return Color.set_color(Color.GREEN, text)


def yellow(text: str) -> str:
    return Color.set_color(Color.YELLOW, text)


def purple(text: str) -> str:
    return Color.set_color(Color.PURPLE, text)


def black(text: str) -> str:
    return Color.set_color(Color.BLACK, text)



# -*- coding: utf-8 -*-
import json
import os
from shutil import rmtree
import setuptools

# ======================================================================================================================
# Fill in this information for each package.
# ======================================================================================================================

# Edit these.
AUTHOR = "Jakrin Juangbhanich"
EMAIL = "juangbhanich.k@gmail.com"
DESCRIPTION = "Package Description."
REPO = "https://github.com/krinj"

# Project Defaults.
PACKAGE_SRC = "src"

# ======================================================================================================================
# Discover the core package.
# ======================================================================================================================

# Work out the sources package.
src_paths = os.listdir(PACKAGE_SRC)
if "__pycache__" in src_paths:
    src_paths.remove("__pycache__")  # Make sure we don't include this by accident.
src_paths.remove("__init__.py")  # Make sure we don't include this by accident.

if len(src_paths) != 1:
    raise Exception(f"Failed to build: Source directory '{PACKAGE_SRC}' must contain exactly one Python package. "
                    f"Instead, it contains {len(src_paths)}: {src_paths}")

PACKAGE_NAME = src_paths[0]
PACKAGE_PATH = os.path.join(PACKAGE_SRC, PACKAGE_NAME)

print(f"Package Name Discovered: {PACKAGE_NAME}")

# ======================================================================================================================
# Automatic Package Setup Script.
# ======================================================================================================================

# Bump the version.
with open("version", "r") as f:
    current_version = f.readline()
    versions = current_version.split(".")
    versions[-1] = str(int(versions[-1]) + 1)
    new_version = ".".join(versions)

with open("version", "w") as f:
    f.write(new_version)

print("Bumping Version Number: {} -> {}".format(current_version, new_version))

with open("version", "r") as f:
    VERSION = f.readline()


def copy_version_to_package(path):
    """ Copy the single source of truth version number into the package as well. """
    init_file = os.path.join(path, "__init__.py")
    with open(init_file, "r") as original_file:
        lines = original_file.readlines()

    with open(init_file, "w") as new_file:
        for line in lines:
            if "__version__" not in line:
                new_file.write(line)
            else:
                new_file.write("__version__ = \"{}\"\n".format(VERSION))


copy_version_to_package(PACKAGE_PATH)

with open("long_description.md", "r") as f:
    long_description = f.read()

packages = setuptools.find_packages(PACKAGE_SRC)
print(f"Packages Discovered: {packages}")

with open("requirements.txt", "r") as f:
    requirement_packages = [line.strip('\n') for line in f.readlines()]
print(f"Requirements: {requirement_packages}")

setuptools.setup(
    author=AUTHOR,
    author_email=EMAIL,
    name=PACKAGE_NAME,
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type="text/markdown",
    version=VERSION,
    url=REPO,
    packages=packages,
    package_dir={PACKAGE_NAME: PACKAGE_PATH},
    install_requires=requirement_packages,
    classifiers=[
        "Programming Language :: Python :: 3.7"
    ],
)


def load_credentials():
    CREDENTIALS_PATH = "pypi_credentials.json"
    if not os.path.exists(CREDENTIALS_PATH):
        with open(CREDENTIALS_PATH, "w") as f:
            json.dump({"user": "username", "pass": "password"}, f)
        print(f"No credentials detected. Please enter your credentials in: {CREDENTIALS_PATH}")
        exit(0)

    with open(CREDENTIALS_PATH, "r") as f:
        credentials = json.load(f)

    return credentials["user"], credentials["pass"]


def upload_distribution():
    repo_user, repo_pass = load_credentials()
    command = f"twine upload -u {repo_user} -p {repo_pass} dist/*"
    os.system(command)


upload_distribution()


def remove_artifacts(path):
    if os.path.exists(path):
        rmtree(path)
        print(f"Removed: {path}")


remove_artifacts("build")
remove_artifacts("dist")
remove_artifacts(f"{PACKAGE_NAME}.egg-info")
